package com.inneractive.api.ads.mediations;

import android.app.Activity;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.Log;

import com.fyber.inneractive.sdk.external.InneractiveUnitController.AdDisplayError;
import com.google.android.gms.ads.AdRequest;
import com.fyber.inneractive.sdk.external.InneractiveAdManager;
import com.fyber.inneractive.sdk.external.InneractiveAdRequest;
import com.fyber.inneractive.sdk.external.InneractiveAdSpot;
import com.fyber.inneractive.sdk.external.InneractiveAdSpotManager;
import com.fyber.inneractive.sdk.external.InneractiveErrorCode;
import com.fyber.inneractive.sdk.external.InneractiveFullscreenAdEventsListener;
import com.fyber.inneractive.sdk.external.InneractiveFullscreenUnitController;
import com.fyber.inneractive.sdk.external.InneractiveFullscreenVideoContentController;
import com.fyber.inneractive.sdk.external.InneractiveMediationDefs;
import com.fyber.inneractive.sdk.external.InneractiveMediationName;
import com.fyber.inneractive.sdk.external.InneractiveUserConfig;
import com.fyber.inneractive.sdk.external.VideoContentListener;
import com.mopub.common.LifecycleListener;
import com.mopub.common.MediationSettings;
import com.mopub.common.MoPubReward;
import com.mopub.common.logging.MoPubLog;
import com.mopub.mobileads.CustomEventRewardedVideo;
import com.mopub.mobileads.MoPubErrorCode;
import com.mopub.mobileads.MoPubRewardedVideoManager;

import java.util.Hashtable;
import java.util.Map;

// Tested with Inneractive SDK 7.0.0
public class InneractiveRewardedVideoForMopub extends CustomEventRewardedVideo {

    // Set your Inneractive's spot ID!
    private static String SAMPLE_REWARDED_SPOT_ID = "Set_Your_Inneractive_Spot_Id"; //for example, inneractive's spot id for testing: "150949"

    InneractiveAdSpot mRewardedSpot;
    Activity mParentActivity;
    private boolean mRewarded = false;

    @Override
    protected LifecycleListener getLifecycleListener() {
        return null;
    }

    @Override
    protected String getAdNetworkId() {
        return SAMPLE_REWARDED_SPOT_ID;
    }

    @Override
    protected void onInvalidate() {
    }

    @Override
    protected boolean checkAndInitializeSdk(@NonNull Activity launcherActivity,
            @NonNull Map<String, Object> localExtras,
            @NonNull Map<String, String> serverExtras)
            throws Exception {

        // TODO: Nothing to do here. The SDK is initialized in the Sample's main activity. Is this the right way to go?

        return true;
    }

    @Override
    protected void loadWithSdkInitialized(@NonNull Activity activity,
            @NonNull Map<String, Object> localExtras,
            @NonNull Map<String, String> serverExtras)
            throws Exception {
        Log.d(InneractiveMediationDefs.INNERACTIVE_MEDIATION_SAMPLE_APP_TAG , InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED + " - loading rewarded");

        // Set variables from MoPub console.
        String resultSpotId = null;
        String resultKeywords = null;

        if (serverExtras != null) {
            resultSpotId = serverExtras.get(InneractiveMediationDefs.REMOTE_KEY_SPOT_ID);
            if (!TextUtils.isEmpty(serverExtras.get(InneractiveMediationDefs.REMOTE_KEY_KEYWORDS))) {
                resultKeywords = serverExtras.get(InneractiveMediationDefs.REMOTE_KEY_KEYWORDS);
            }
        }

        //Make sure we have a spotId from the request.
        if (TextUtils.isEmpty(resultSpotId)){
            resultSpotId = SAMPLE_REWARDED_SPOT_ID;
            if (TextUtils.isEmpty(resultSpotId)) {
                MoPubRewardedVideoManager.onRewardedVideoLoadFailure(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID, MoPubErrorCode.ADAPTER_CONFIGURATION_ERROR);
                return;
            }
        }

        InneractiveUserConfig.Gender gender = null;
        int age = 0;
        String zipCode = null;
        if (localExtras != null) {
            
			/* Set keywords variable as defined on MoPub console, you can also define keywords with setlocalExtras in IaMediationActivity class.
			in case the variable is not initialized, the variable will not be in use */
            if (localExtras.containsKey(InneractiveMediationDefs.KEY_KEYWORDS) && TextUtils.isEmpty(resultKeywords)) {
                resultKeywords = (String) localExtras.get(InneractiveMediationDefs.KEY_KEYWORDS);
            }
            
			/* Set the age variable as defined on IaMediationActivity class.   
			in case the variable is not initialized, the variable will not be in use */
            if (localExtras.containsKey(InneractiveMediationDefs.KEY_AGE)) {
                try {
                    age = Integer.valueOf(localExtras.get(InneractiveMediationDefs.KEY_AGE).toString());
                } catch (NumberFormatException e) {
                    Log.d(InneractiveMediationDefs.INNERACTIVE_MEDIATION_SAMPLE_APP_TAG, InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED + " Invalid Age");
                }
            }

			/* Set the gender variable as defined on IaMediationActivity class.   
			in case the variable is not initialized, the variable will not be in use */
            if (localExtras.containsKey(InneractiveMediationDefs.KEY_GENDER)) {
                String  genderStr = localExtras.get(InneractiveMediationDefs.KEY_GENDER).toString();
                if (genderStr.equals(InneractiveMediationDefs.GENDER_MALE)) {
                    gender = InneractiveUserConfig.Gender.MALE;
                } else if (genderStr.equals(InneractiveMediationDefs.GENDER_FEMALE)) {
                    gender = (InneractiveUserConfig.Gender.FEMALE);
                }
            }

			/* Set zipCode variable as defined on IaMediationActivity class.   
			in case the variable is not initialized, the variable will not be in use */
            if (localExtras.containsKey(InneractiveMediationDefs.KEY_ZIPCODE)) {
                zipCode = (String) localExtras.get(InneractiveMediationDefs.KEY_ZIPCODE);
            }
        }

        if (mRewardedSpot != null) {
            mRewardedSpot.destroy();
        }

        mRewardedSpot = InneractiveAdSpotManager.get().createSpot();
        // Set your mediation name
        mRewardedSpot.setMediationName(InneractiveMediationName.MOPUB);

        InneractiveFullscreenUnitController fullscreenUnitController = new InneractiveFullscreenUnitController();
        mRewardedSpot.addUnitController(fullscreenUnitController);

        InneractiveAdRequest request = new InneractiveAdRequest(resultSpotId);
        request.setUserParams( new InneractiveUserConfig()
                .setGender(gender)
                .setZipCode(zipCode)
                .setAge(age));
        if (resultKeywords != null && resultKeywords.length() > 0) {
            request.setKeywords(resultKeywords.toString());
        }

        // Load ad
        mRewardedSpot.setRequestListener(new InneractiveAdSpot.RequestListener() {

            /**
             * Called by Inneractive when an interstitial is ready for display
             * @param adSpot Spot object
             */
            @Override
            public void onInneractiveSuccessfulAdRequest(InneractiveAdSpot adSpot) {
                Log.d(InneractiveMediationDefs.INNERACTIVE_MEDIATION_SAMPLE_APP_TAG, InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED + " - inneractiveInterstitialLoaded");
                MoPubRewardedVideoManager.onRewardedVideoLoadSuccess(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID);
            }

            /**
             * Called by Inneractive an interstitial fails loading
             * @param adSpot Spot object
             * @param errorCode the failure's error.
             */
            @Override
            public void onInneractiveFailedAdRequest(InneractiveAdSpot adSpot, InneractiveErrorCode errorCode) {
                Log.d(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "Failed loading interstitial! with error: " + errorCode);
                if (errorCode == InneractiveErrorCode.CONNECTION_ERROR) {
                    MoPubRewardedVideoManager.onRewardedVideoLoadFailure(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID, MoPubErrorCode.NO_CONNECTION);
                } else if  (errorCode == InneractiveErrorCode.CONNECTION_TIMEOUT) {
                    MoPubRewardedVideoManager.onRewardedVideoLoadFailure(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID, MoPubErrorCode.NETWORK_TIMEOUT);
                } else if (errorCode == InneractiveErrorCode.NO_FILL) {
                    MoPubRewardedVideoManager.onRewardedVideoLoadFailure(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID, MoPubErrorCode.NO_FILL);
                } else {
                    MoPubRewardedVideoManager.onRewardedVideoLoadFailure(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID, MoPubErrorCode.SERVER_ERROR);
                }
            }
        });

        mRewardedSpot.requestAd(request);

        mParentActivity = activity;
    }

    @Override
    protected boolean hasVideoAvailable() {
        return mRewardedSpot != null && mRewardedSpot.isReady();
    }

    @Override
    protected void showVideo() {
        Log.d(InneractiveMediationDefs.INNERACTIVE_MEDIATION_SAMPLE_APP_TAG , InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED + " - showInterstitial");
        // check if the ad is ready
        if (mRewardedSpot != null && mRewardedSpot.isReady()) {

            InneractiveFullscreenUnitController fullscreenUnitController = (InneractiveFullscreenUnitController)mRewardedSpot.getSelectedUnitController();
            fullscreenUnitController.setEventsListener(new InneractiveFullscreenAdEventsListener() {

                /**
                 * Called by Inneractive when an interstitial ad activity is closed
                 * @param adSpot Spot object
                 */
                @Override
                public void onAdDismissed(InneractiveAdSpot adSpot) {
                    MoPubRewardedVideoManager.onRewardedVideoClosed(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID);
                    Log.i(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "onAdDismissed");
                    MoPubRewardedVideoManager.onRewardedVideoCompleted(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID, mRewarded ? MoPubReward.success(MoPubReward.NO_REWARD_LABEL, MoPubReward.DEFAULT_REWARD_AMOUNT) : MoPubReward.failure());
                }

                /**
                 * Called by Inneractive when an interstitial ad activity is shown
                 * @param adSpot Spot object
                 */
                @Override
                public void onAdImpression(InneractiveAdSpot adSpot) {
                    MoPubRewardedVideoManager.onRewardedVideoStarted(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID);
                    Log.i(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "onAdImpression");
                }

                /**
                 * Called by Inneractive when an interstitial ad is clicked
                 * @param adSpot Spot object
                 */
                @Override
                public void onAdClicked(InneractiveAdSpot adSpot) {
                    MoPubRewardedVideoManager.onRewardedVideoClicked(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID);
                    Log.i(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "onAdClicked");
                }

                /**
                 * Called by Inneractive when an interstitial ad opened an external application
                 * @param adSpot Spot object
                 */
                @Override
                public void onAdWillOpenExternalApp(InneractiveAdSpot adSpot) {
                    Log.d(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "inneractiveAdWillOpenExternalApp");
                    // Don't call the onLeaveApplication() API since it causes a false Click event on MoPub
                }

                /**
                 * Called when an ad has entered an error state, this will only happen when the ad is being shown
                 * @param adSpot the relevant ad spot
                 */
                @Override
                public void onAdEnteredErrorState(InneractiveAdSpot adSpot, AdDisplayError error) {
                    Log.i(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "onAdEnteredErrorState - " + error.getMessage());
                }

                /**
                 * Called by Inneractive when Inneractive's internal browser, which was opened by this interstitial, was closed
                 * @param adSpot Spot object
                 */
                @Override
                public void onAdWillCloseInternalBrowser(InneractiveAdSpot adSpot) {
                    Log.d(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "inneractiveInternalBrowserDismissed");
                }
            });

            // Add video content controller, for controlling video ads
            InneractiveFullscreenVideoContentController videoContentController = new InneractiveFullscreenVideoContentController();
            videoContentController.setEventsListener(new VideoContentListener() {
                @Override
                public void onProgress(int totalDurationInMsec, int positionInMsec) {
                    Log.d(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "Interstitial: Got video content progress: total time = " + totalDurationInMsec + " position = " + positionInMsec);
                }

                /**
                 * Called by inneractive when an Intersititial video ad was played to the end
                 * <br>Can be used for incentive flow
                 * <br>Note: This event does not indicate that the interstitial was closed
                 */
                @Override
                public void onCompleted() {
                    mRewarded = true;
                    Log.d(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "Interstitial: Got video content completed event");
                }

                @Override
                public void onPlayerError() {
                    Log.d(InneractiveMediationDefs.IA_LOG_FOR_ADMOB_INTERSTITIAL, "Interstitial: Got video content play error event");
                    MoPubRewardedVideoManager.onRewardedVideoPlaybackError(InneractiveRewardedVideoForMopub.class, SAMPLE_REWARDED_SPOT_ID, MoPubErrorCode.VIDEO_PLAYBACK_ERROR);
                }
            });

            // Now add the content controller to the unit controller
            fullscreenUnitController.addContentController(videoContentController);

            fullscreenUnitController.show((Activity)mParentActivity);
        } else {
            Log.d(InneractiveMediationDefs.IA_LOG_FOR_MOPUB_REWARDED, "The Interstitial ad is not ready yet.");
        }
    }

}
